----------------------------------------------------------------------------------
--  File: top.vhd
----------------------------------------------------------------------------------
--  Author: Jordan Retz
--  CopyRight (c) 2013 Digilent Inc.
----------------------------------------------------------------------------------
--  Revision History:
--  	07/25/2013	Created (Jordan Retz)
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;
use ieee.numeric_std.all;

library UNISIM;
use UNISIM.VComponents.all;


----------------------------------------------------------------------------------
--	Title: top Entity
----------------------------------------------------------------------------------
--  Inputs: 
--  Outputs: 
----------------------------------------------------------------------------------
--  Description:
--  	
----------------------------------------------------------------------------------
entity top is
	Port ( 
			-- FPGA_GCLK, 8MHz
			CLK				: in STD_LOGIC;
			
			-- FPGA_LFC, 1 Hz
			CLK_LFC			: in STD_LOGIC;
			
			-- LED outputs
			LED_0				: out STD_LOGIC;
			LED_1				: out STD_LOGIC;
			LED_2				: out STD_LOGIC;
			LED_3				: out STD_LOGIC;
			
			-- BTN inputs
			BTN_0				: in STD_LOGIC;
			BTN_1				: in STD_LOGIC;
			
			-- DEPP address strobe (ASTB) and DEPP data strobe (DSTB)
         DEPP_ASTB 		: in  STD_LOGIC;
         DEPP_DSTB 		: in  STD_LOGIC;
			  
			-- DEPP write enable (write operation = 0, read operation = 1)
         DEPP_WRITE 		: in  STD_LOGIC;
			  
			-- DEPP logic ready 
			DEPP_WAIT		: out STD_LOGIC;
			  
			-- DEPP bi-directional data bus. Direction controlled by DEPP_WRITE
         DBUS 				: inout  STD_LOGIC_VECTOR (7 downto 0);
			
			-- connected to physical I/O pad
			PORTA 			: in     STD_LOGIC_VECTOR (7 downto 0);
			PORTB 			: inout  STD_LOGIC_VECTOR (7 downto 0);
			PORTC 			: inout	STD_LOGIC_VECTOR (6 downto 0);
			PORTD 			: in		STD_LOGIC_VECTOR (7 downto 0);
			PORTE 			: inout  STD_LOGIC_VECTOR (7 downto 0);	
			PORTF				: out    STD_LOGIC_VECTOR (5 downto 0);
			EXT_CLK			: in     STD_LOGIC
		);
end top;

-------------------------------------------------------
-------------------------------------------------------
-- Architecture Start
-------------------------------------------------------
-------------------------------------------------------

architecture Behavioral of top is

-------------------------------------------------------
-- Sub module interconnect signals
-------------------------------------------------------
--signal port_a_top 		: STD_LOGIC_VECTOR (7 downto 0);
--signal port_b_top 		: STD_LOGIC_VECTOR (7 downto 0);
--signal port_c_top 		: STD_LOGIC_VECTOR (6 downto 0);
--signal port_d_top 		: STD_LOGIC_VECTOR (7 downto 0);
--signal port_e_top 		: STD_LOGIC_VECTOR (7 downto 0);
--signal port_f_top 		: STD_LOGIC_VECTOR (6 downto 0);

signal bus_cntrl_to_io 	: STD_LOGIC_VECTOR (7 downto 0);
signal bus_io_to_cntrl 	: STD_LOGIC_VECTOR (7 downto 0);

signal bus_addr			: STD_LOGIC_VECTOR (3 downto 0);

signal hs 					: STD_LOGIC;

signal BCK_CLK				: STD_LOGIC;

signal IND					: STD_LOGIC_VECTOR (3 downto 0);

signal volume				: STD_LOGIC_VECTOR (6 downto 0);

-------------------------------------------------------
-- 50 MHz clock
-------------------------------------------------------
signal clk_50 				: STD_LOGIC;

-------------------------------------------------------
-- Component Declaration
-------------------------------------------------------
component port_io
	Port 	( 	CLK 				: in    STD_LOGIC;
				ADDRESS 			: in    STD_LOGIC_VECTOR (3 downto 0);
				RW					: in    STD_LOGIC;
				HS					: in    STD_LOGIC;
				REG_OUT 			: out   STD_LOGIC_VECTOR (7 downto 0);
				REG_IN	 		: in    STD_LOGIC_VECTOR (7 downto 0);
				PORTA 			: inout STD_LOGIC_VECTOR (7 downto 0);
				PORTB 			: inout STD_LOGIC_VECTOR (7 downto 0);
				PORTC 			: inout STD_LOGIC_VECTOR (6 downto 0);
				PORTD 			: inout STD_LOGIC_VECTOR (7 downto 0);
				PORTE 			: inout STD_LOGIC_VECTOR (7 downto 0);	
				PORTF 			: inout STD_LOGIC_VECTOR (6 downto 0)
		 );
end component;

component depp_cntrl
	Port	( 	CLK 				: in    STD_LOGIC;
				DEPP_ASTB 		: in    STD_LOGIC;
				DEPP_DSTB 		: in    STD_LOGIC;
				DEPP_WRITE 		: in    STD_LOGIC;
				DEPP_WAIT		: out   STD_LOGIC;
				DBUS 				: inout STD_LOGIC_VECTOR (7 downto 0);
				INTERNAL_DOUT 	: out   STD_LOGIC_VECTOR (7 downto 0);
				INTERNAL_DIN 	: in    STD_LOGIC_VECTOR (7 downto 0);
				INTERNAL_AOUT 	: out   STD_LOGIC_VECTOR (3 downto 0);
				INTERNAL_HS		: out   STD_LOGIC
			);
end component;

component clk_gen_50MHz is
port	( 		CLK_IN1        : in     STD_LOGIC;
				CLK_OUT1       : out    STD_LOGIC
		);
end component;

component i2s_top
	Port	( 	CLK 				: in    STD_LOGIC;
				RST				: in    STD_LOGIC;
				I2S_DATA_I		: in    STD_LOGIC;
				I2S_LRCK_I 		: in    STD_LOGIC;
				I2S_BCK_I 		: in    STD_LOGIC;
				I2S_DATA_O		: out   STD_LOGIC;
				I2S_LRCK_O 		: out   STD_LOGIC;
				I2S_BCK_O 		: out   STD_LOGIC;
				CTRL				: in	  STD_LOGIC;
				IND				: out	  STD_LOGIC_VECTOR(3 downto 0);
				Volume			: in    STD_LOGIC_VECTOR(6 downto 0)
			);
end component;

component BCK_clk_gen_24576
	Port	(	-- Clock in ports
				CLK_IN1        : in     std_logic;
				-- Clock out ports
				CLK_OUT1       : out    std_logic
			 );
end component;

component rotary
    Port ( CLK 				: in		STD_LOGIC;
           A 					: in		STD_LOGIC;
           B 					: in		STD_LOGIC;
			  value				: out std_logic_vector(6 downto 0) --7 bit value out
			);
end component;

-------------------------------------------------------
-- Behavioral Begin
-------------------------------------------------------
begin
	 
-------------------------------------------------------
-- Component Instantiation
-------------------------------------------------------
CLK_GEN: clk_gen_50MHz port map (
										CLK_IN1 			=> CLK,
										CLK_OUT1 		=> clk_50
										  );
			
BCK_CLK_GEN: BCK_clk_gen_24576 port map (
										CLK_IN1 			=> EXT_CLK,
										CLK_OUT1 		=> BCK_CLK
								 );
										  
PORTIO: port_io port map (
										CLK 				=> clk_50,
										ADDRESS 			=> bus_addr,
										RW 				=> DEPP_WRITE,
										HS 				=> hs,
										REG_OUT 			=> bus_io_to_cntrl,
										REG_IN 			=> bus_cntrl_to_io,
										PORTA 			=> open,
										PORTB 			=> PORTB,
										PORTC 			=> PORTC,
										PORTD 			=> open, --PORTD,
										PORTE 			=> PORTE,
										PORTF 			=> open
								  );
									
DEPPCNTRL: depp_cntrl port map	(  
										CLK 				=> clk_50,
										DEPP_ASTB 		=> DEPP_ASTB,
										DEPP_DSTB 		=> DEPP_DSTB,
										DEPP_WRITE 		=> DEPP_WRITE,
										DEPP_WAIT 		=> DEPP_WAIT,
										DBUS 				=> DBUS,
										INTERNAL_DOUT 	=> bus_cntrl_to_io,
										INTERNAL_DIN 	=> bus_io_to_cntrl,
										INTERNAL_AOUT 	=> bus_addr,
										INTERNAL_HS 	=> hs
											);

IS2TOP: i2s_top port map (
										CLK 				=> BCK_CLK,
										RST				=> BTN_1,
										I2S_DATA_I 		=> PORTA(0),
										I2S_LRCK_I 		=> PORTA(1),
										I2S_BCK_I 		=> PORTA(3),
										I2S_DATA_O 		=> PORTF(5),
										I2S_LRCK_O 		=> PORTF(4),
										I2S_BCK_O 		=> PORTF(3),
										CTRL				=> BTN_0,
										IND				=> IND,
										Volume			=> volume
								  );
	 
ROTARYTOP: rotary port map (
										CLK 				=> clk_50,
										A 					=> PORTD(4),
										B 					=> PORTD(2),
										value				=> volume
								   );
									
-------------------------------------------------------
-- LED/BTN Testing
-------------------------------------------------------
--LED_3 <= CLK_LFC;
--LED_2 <= not CLK_LFC;

--LED_0 <= BTN_0 when BTN_0 = '1' else CLK_LFC;
--LED_1 <= BTN_1 when BTN_1 = '1' else not CLK_LFC;

LED_0 <= IND(0);
--LED_1 <= IND(1);
LED_1 <= volume(0);
LED_2 <= IND(2);
--LED_3 <= IND(3);
LED_3 <= volume(6);

--force unused outputs
PORTF(2 downto 0) <= "000";

end Behavioral;
